package com.usky.dxtop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.mapper.DreUserMapper;
import com.usky.dxtop.model.DreUser;
import com.usky.dxtop.model.Staff;
import com.usky.dxtop.service.DreUserService;
import com.usky.dxtop.service.vo.DreUserVO;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yq
 * @since 2022-08-15
 */
@Service
public class DreUserServiceImpl extends ServiceImpl<DreUserMapper, DreUser> implements DreUserService {

    @Override
    public CommonPage<DreUserVO> page(Integer current, Integer size,String userName,Boolean isSend) {
        IPage<DreUserVO> page = new Page<>(current, size);
        page = baseMapper.page(page, userName,isSend);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public void addList(List<DreUser> list) {
        if (CollectionUtils.isEmpty(list)){
            throw new CustomException("请选择要关联的信息");
        }
        checkIsSend(list);
        List<DreUser> removeList = list.stream()
                .filter(dreUser -> !dreUser.isDelFlag()).collect(Collectors.toList());

        List<DreUser> addList = list.stream()
                .filter(dreUser -> dreUser.isDelFlag() && null == dreUser.getId()).collect(Collectors.toList());

        List<DreUser> updateList = list.stream()
                .filter(dreUser -> null != dreUser.getId()).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(removeList)){
            LambdaQueryWrapper<DreUser> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.in(DreUser::getUserId,removeList.stream().map(DreUser::getUserId).collect(Collectors.toList()));
            this.remove(queryWrapper);
        }
        if (CollectionUtils.isNotEmpty(addList)){
            this.saveBatch(addList);
        }
        if (CollectionUtils.isNotEmpty(updateList)){
            this.updateBatchById(updateList);
        }
    }


    public void checkIsSend(List<DreUser> list){
        list = list.stream().filter(dreUser -> 1 == dreUser.getIsSend() && StringUtils.isBlank(dreUser.getOpenId())).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(list)){
            StringBuilder msg = new StringBuilder();
            msg.append("配送员-");
            for (DreUser dreUser:list) {
                msg.append(dreUser.getUserName()).append(",");
            }
            msg.append("--未绑定openId,请登录公众号开启");
            throw new CustomException(msg.toString());
        }
    }

    @Override
    public List<DreUserVO> list(String name, Boolean isSend) {
        return baseMapper.list(name,isSend);
    }

    @Override
    public void updateOpenId(Staff staff) {
        if (StringUtils.isNotBlank(staff.getOpenId())){
            LambdaQueryWrapper<DreUser> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.eq(DreUser::getUserId,staff.getSId());
            List<DreUser> list = this.list(queryWrapper);
            if (CollectionUtils.isNotEmpty(list)){
                list.forEach(d -> d.setOpenId(staff.getOpenId()));
                this.updateBatchById(list);
            }
        }
    }
}
